<?php

namespace Drupal\ww_publish;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\field\FieldConfigInterface;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;

class Article {

  /**
   * Metadata of the article received from WoodWing Studio.
   *
   * @var \Drupal\ww_publish\Metadata
   */
  private $articleMetadata;

  /**
   * Article data received from WoodWing Studio.
   *
   * @var Object
   */
  private $articleData;

  /**
   * Prepared files.
   *
   * @var array
   */
  private $preparedFiles;

  /**
   * Configuration of the ww_publish module.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $config;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Content type in which the article should be saved.
   *
   * @var \Drupal\node\Entity\NodeType
   */
  private $contentType;

  /**
   * WoodWing article ID.
   *
   * @var int
   */
  private $id;

  /**
   * WoodWing Studio article ID field.
   *
   * @var string
   */
  private $articleIdField;

  /**
   * Paragraph field.
   *
   * @var string
   */
  private $paragraphField;

  /**
   * The target node which will be created or updated.
   *
   * @var \Drupal\node\Entity\Node
   */
  private $node;

  /**
   * Constructor.
   *
   * @param \Drupal\ww_publish\Metadata $article_metadata
   *   Article metadata.
   * @param Object $article_data
   *   Article data.
   * @param array $prepared_files
   *   Prepared files.
   * @param \Drupal\Core\Config\Config $config
   *   Configuration of the ww_publish module.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct($article_metadata, $article_data, $prepared_files, $config, $logger) {
    $this->articleMetadata = $article_metadata;
    $this->articleData = $article_data;
    $this->preparedFiles = $prepared_files;
    $this->config = $config;
    $this->logger = $logger;
    $this->contentType = $this->getContentType();
    $this->id = $this->getId();
    $this->articleIdField = $this->getContentTypeSetting('article_id_field');
    $this->paragraphField = $this->getContentTypeSetting('paragraph_field');
  }

  /**
   * Publish a new node or update an existing node.
   *
   * @return bool
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function publish(): bool {
    if ($this->config->get('debug_mode'))
      $this->logger->debug('Publish!');

    $node = $this->findRelatedNode();

    if ($node) {
      $this->node = $node;
      $this->update();
    } else if ($node === 0) {
      $this->create();
    } else {
      $this->logger->error('There are more than one nodes with the same WoodWing Studio article ID.');
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Create a new node.
   *
   * @return bool
   * @throws \Drupal\Core\Entity\EntityStorageException|\Drupal\Core\TypedData\Exception\ReadOnlyException
   */
  private function create(): bool {
    $title = $this->getTitle();
    try {
      $this->node = Node::create([
        'type' => $this->contentType->get('type'),
        'title' => $title,
        $this->articleIdField => $this->id,
      ]);
      $this->updateFields();
      $this->node->setOwnerId($this->articleMetadata->getAuthor()->id());

      // Test if during updating the fields a related node was created.
      $relatedNode = $this->findRelatedNode();

      // Prevent creation of a duplicated node, when Amazon sends more request at the same time.
      if ($relatedNode === 0) {
        if ($this->config->get('debug_mode'))
          $this->logger->debug('Node to be created: <pre><code>' . print_r($this->node->toArray(), TRUE) . '</code></pre>');
        $this->node->save();
        $this->logger->notice('The following article has been created: ' . $title . ' (WW ID: ' . $this->id . ', Drupal ID: ' . $this->node->id() . ')');
      } else if ($relatedNode) {
        $this->logger->error('The node already exists. Probably has Amazon sent more requests at the same time.');
      } else {
        $this->logger->error('There are more than one nodes with the same WoodWing Studio article ID.');
      }
    }
    catch (EntityStorageException $e) {
      $this->logger->error('A node could not be created: <pre><code>' . print_r($e, TRUE) . '</code></pre>');
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Update an existing node.
   *
   * @return bool
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function update(): bool {
    if ($this->node instanceof NodeInterface) {
      $title = $this->getTitle();
      try {
        $this->node->set('title', $title);

        $this->updateFields();
        $this->node->setRevisionUserId($this->articleMetadata->getAuthor()->id());

        if ($this->config->get('debug_mode'))
          $this->logger->debug('Node to be updated: <pre><code>' . print_r($this->node->toArray(), TRUE) . '</code></pre>');

        $this->node->save();
        $this->logger->notice('The following article has been updated: ' . $title . ' (WW ID: ' . $this->id . ', Drupal ID: ' . $this->node->id() . ')');
      }
      catch (\Exception $e) {
        $this->logger->error('The node could not be updated: <pre><code>' . print_r($e, TRUE) . '</code></pre>');
        return FALSE;
      }
    }

    return TRUE;
  }

  /**
   * Update fields.
   *
   * @throws \Drupal\Core\TypedData\Exception\ReadOnlyException
   */
  private function updateFields() {
    // Define an empty error messages field.
    $messagesFieldName = $this->config->get('error_messages_field');
    $messagesField = $this->node->hasField($messagesFieldName) ?
      $this->node->get($messagesFieldName)->setValue([]) : NULL;

    $fields = $this->getContentTypeFields();
    foreach ($fields as $fieldName => $fieldConfig) {
      $contentStationIdentifier = $fieldConfig->getThirdPartySetting('ww_publish', 'content_station_identifier');
      if ($contentStationIdentifier) {
        $field = new Field($fieldConfig, $contentStationIdentifier, $this->articleMetadata ,$this->articleData, $this->preparedFiles, $this->articleIdField, $this->config, $this->logger);
        $articleData = $field->getArticleData($messagesField);
        if ($this->config->get('debug_mode'))
          $this->logger->debug($fieldName . ': <pre><code>' . print_r($articleData, TRUE) . '</code></pre>');
        $this->node->set($fieldName, $articleData);
      }
      if ($fieldName === $this->paragraphField) {
        $paragraphField = new ParagraphField($fieldConfig, $this->articleMetadata, $this->articleData->data->content, $this->preparedFiles, $this->articleIdField, $this->config, $this->logger);
        $this->node->set($fieldName, $paragraphField->getArticleData($messagesField));
      }
    }

    if ($messagesField)
      $this->node->get($messagesFieldName)->setValue($messagesField);
  }

  /**
   * Get content type fields.
   *
   * @return array
   */
  private function getContentTypeFields(): array {
    $contentType = $this->contentType->get('type');

    $entityManager = \Drupal::service('entity.manager');
    $fields = [];

    if(!empty($contentType)) {
      $fields = array_filter(
        $entityManager->getFieldDefinitions('node', $contentType),
        function ($field_definition) {
          return
            $field_definition instanceof FieldConfigInterface;
        }
      );
    }

    return $fields;
  }

  /**
   * Get the target content type.
   *
   * @return mixed
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getContentType() {
    $contentTypeMachineName = FALSE;
    foreach ($this->articleMetadata->getMetadata()->MetaData->ExtraMetaData as $extraMetaDataField) {
      if ($extraMetaDataField->Property == $this->config->get('content_type_field')) {
        if ($this->config->get('debug_mode'))
          $this->logger->debug('Target content type: <pre><code>' . print_r($extraMetaDataField->Values[0], TRUE) . '</code></pre>');
        $contentTypeMachineName = $extraMetaDataField->Values[0];
        break;
      }
    }
    if ($contentTypeMachineName) {
      // Get all content types.
      try {
        $types = \Drupal::entityTypeManager()
          ->getStorage('node_type')
          ->loadMultiple();
      } catch (InvalidPluginDefinitionException $e) {
        $this->logger->error('Could not get content types.');
        return FALSE;
      } catch (PluginNotFoundException $e) {
        $this->logger->error('Could not get content types.');
        return FALSE;
      }

      // Test if the target content type exists in Drupal.
      if ($this->config->get('debug_mode'))
        $this->logger->debug('Existing content types: <pre><code>' . print_r($types, TRUE) . '</code></pre>');
      if ($types[$contentTypeMachineName]) {
        // TODO: Test if the content type is correctly configured.
        if ($this->config->get('debug_mode'))
          $this->logger->debug('The target content type exists in Drupal and is correctly configured.');

        return \Drupal::entityTypeManager()
          ->getStorage('node_type')
          ->load($contentTypeMachineName);

      } else {
        $this->logger->error('The content type does not exist in Drupal.');
        return FALSE;
      }
    } else {
      $this->logger->error('The content type field could not be found in the metadata.');
      return FALSE;
    }
  }

  /**
   * Get the ID of the WoodWing Studio article.
   *
   * @return mixed
   */
  private function getId() {
    if ($this->articleMetadata->getMetadata()->MetaData->BasicMetaData->ID) {
      if ($this->config->get('debug_mode'))
        $this->logger->debug('WoodWing Studio article ID: <pre><code>' . print_r($this->articleMetadata->getMetadata()->MetaData->BasicMetaData->ID, TRUE) . '</code></pre>');
      return (int)$this->articleMetadata->getMetadata()->MetaData->BasicMetaData->ID;
    } else {
      $this->logger->error('No article ID found in metadata.');
      return FALSE;
    }
  }

  /**
   * Get the title of the WoodWing Studio article.
   *
   * @return mixed
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function getTitle() {
    // Get the WoodWing Studio identifier for the title from content type configuration.
    $contentStationIdentifier = $this->contentType->getThirdPartySetting('ww_publish', 'content_station_identifier_title');

    $contentType = $this->contentType->get('type');

    $entityFieldManager = \Drupal::service('entity_field.manager');

    // Get field configuration for the field title.
    $fieldConfig = $entityFieldManager->getFieldDefinitions('node', $contentType)['title'];

    if ($contentStationIdentifier && $fieldConfig) {
      $field = new Field($fieldConfig, $contentStationIdentifier, $this->articleMetadata ,$this->articleData, $this->preparedFiles, $this->articleIdField, $this->config, $this->logger);
      $title = $field->getArticleData();

      if (!$this->config->get('html_title')) {
        $title['value'] = strip_tags($title['value']);
      }

      if ($this->config->get('debug_mode'))
        $this->logger->debug('Title: <pre><code>' . print_r($title, TRUE) . '</code></pre>');
      return $title;
    }

    $this->logger->error('No article title found.');
    return FALSE;
  }

  /**
   * Get the ww_publish setting from content type configuration.
   *
   * @param string $settingName
   *   Setting name, e.g.: 'article_id_field', 'paragraph_field'.
   *
   * @return mixed
   */
  private function getContentTypeSetting($settingName) {
    $setting = $this->contentType->getThirdPartySetting('ww_publish', $settingName);

    if ($setting) {
      return $setting;
    } else {
      $this->logger->notice('The content type setting ' . $settingName .' is not set.');
      return NULL;
    }
  }

  /**
   * Find the related node.
   *
   * Returns the found node if one node was found. Returns 0 if no node was
   * found. Returns FALSE, if two or more nodes were founds or in the case of an
   * exception.
   *
   * @return mixed
   */
  private function findRelatedNode() {
    try {
      $nodes = \Drupal::entityTypeManager()
        ->getStorage('node')
        ->loadByProperties(['field_ww_id' => $this->id]);

      // Test if the node exists.
      if (($node = reset($nodes)) && count($nodes) === 1) {
        return $node;
      } else if (count($nodes) === 0) {
        return 0;
      } else {
        $this->logger->error('There are more than one nodes with the same WoodWing Studio article ID.');
      }
    } catch (InvalidPluginDefinitionException $e) {
      $this->logger->error($e->getMessage());
    } catch (PluginNotFoundException $e) {
      $this->logger->error($e->getMessage());
    }
    return FALSE;
  }
}
