<?php

namespace Drupal\ww_publish\Form;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configures WoodWing Studio settings for this site.
 */
class WwPublishSettingsForm extends ConfigFormBase {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a DirectoryForm.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(FileSystemInterface $file_system = NULL) {
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ww_publish_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ww_publish.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('ww_publish.settings');

    // General form.
    $form['#attributes']['autocomplete'] = 'off';

    $form['woodwing'] = [
      '#type' => 'details',
      '#title' => 'WoodWing Studio',
      '#open' => TRUE,
    ];

    $form['woodwing']['studio_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('WoodWing Studio URL'),
      '#description' => $this->t('E.g.: http://wwe10.my-site.org/Aurora/'),
      '#default_value' => $config->get('studio_url'),
    ];

    $form['woodwing']['content_type_field'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Content type field'),
      '#description' => $this->t('WoodWing Studio metadata field name which contains the machine name of the Drupal content type, in which the nodes should be created/updated.'),
      '#default_value' => $config->get('content_type_field'),
      '#required' => TRUE,
    ];

    // Get all users having the permission to create a media.
    $roles = user_role_names(FALSE, 'create media');
    $rids = array_keys($roles);
    $uids = \Drupal::service('entity_type.manager')
      ->getStorage('user')
      ->getQuery()
      ->condition('roles', $rids, 'IN')
      ->execute();
    $users = [];
    foreach ($uids as $uid) {
      $user = User::load($uid);
      $users[$uid] = $user->getAccountName();
    }
    $defaultUser = ($config->get('content_type_field')) ? $config->get('ww_user') : 1;
    $form['woodwing']['ww_user'] = [
      '#type' => 'select',
      '#title' => $this->t('WW User'),
      '#description' => $this->t('This user will be used to save WW contents in Drupal.'),
      '#default_value' => $defaultUser,
      '#options' => $users,
      '#required' => TRUE,
    ];

    $form['woodwing']['ww_user_field'] = [
      '#type' => 'textfield',
      '#title' => $this->t('WW User field'),
      '#maxlength' => 255,
      '#default_value' => $config->get('ww_user_field'),
      '#description' => $this->t('Field in the user entity used to identify the WoodWing user. E.g.: field_ww_user.'),
    ];

    $form['woodwing']['error_messages_field'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Error messages field'),
      '#maxlength' => 255,
      '#default_value' => $config->get('error_messages_field'),
      '#description' => $this->t('Field in the node entity used to save error messages, which will be displayed to the editors. The field should have the type "long plain text" and unlimited values.'),
    ];

    $form['woodwing']['ww_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Path for the WW public folder'),
      '#maxlength' => 255,
      '#default_value' => $config->get('ww_path'),
      '#description' => $this->t('If the path in the file system does not exists, then will be the directory created.'),
      '#required' => TRUE,
    ];

    $form['woodwing']['debug_mode'] = [
      '#type' => 'checkbox',
      '#title' => t('Activate the debug mode.'),
      '#description' => $this->t('Do not use the debug mode on the live site.'),
      '#default_value' => $config->get('debug_mode'),
    ];

    $form['woodwing']['cron_mode'] = [
      '#type' => 'checkbox',
      '#title' => t('Activate the cron mode.'),
      '#description' => $this->t('In cron mode are the Amazon SNS requests saved in the database and the articles are published, when the next cron job runs. Use the cron mode, when the publishing is too slow, what causes that you have multiple Amazon SNS requests when publishing an article.'),
      '#default_value' => $config->get('cron_mode'),
    ];

    $form['woodwing']['sns_delete'] = [
      '#type' => 'number',
      '#title' => t('Number of days, after which Amazon SNS messages are deleted.'),
      '#description' => $this->t('0 = the messages will never be deleted.'),
      '#default_value' => $config->get('sns_delete'),
    ];

    $form['woodwing']['html_title'] = [
      '#type' => 'checkbox',
      '#title' => t('Enable HTML title.'),
      '#description' => $this->t('The module <a href="https://www.drupal.org/project/html_title">HTML Title</a> have to be installed and enabled!'),
      '#default_value' => $config->get('html_title'),
    ];


    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $path = trim($form_state->getValue('ww_path'));
    $config = $this->config('ww_publish.settings');
    $config->set('studio_url', trim($form_state->getValue('studio_url')));
    $config->set('content_type_field', trim($form_state->getValue('content_type_field')));
    $config->set('ww_user', trim($form_state->getValue('ww_user')));
    $config->set('ww_user_field', trim($form_state->getValue('ww_user_field')));
    $config->set('error_messages_field', trim($form_state->getValue('error_messages_field')));
    $config->set('ww_path', $path);
    $config->set('debug_mode', $form_state->getValue('debug_mode'));
    $config->set('cron_mode', $form_state->getValue('cron_mode'));
    $config->set('sns_delete', $form_state->getValue('sns_delete'));
    $config->set('html_title', $form_state->getValue('html_title'));
    $config->save();

    // Create the directory if it does not exist.
    $directory_path = 'public://' . $path;
    $this->fileSystem->prepareDirectory($directory_path, FileSystemInterface::CREATE_DIRECTORY);

    parent::submitForm($form, $form_state);
  }

}
