<?php

namespace Drupal\ww_publish;

use Drupal\Core\Entity\EntityInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\paragraphs\Entity\Paragraph;

class ParagraphField {

  /**
   * Paragraph filed configuration.
   *
   * @var \Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList
   */
  private $fieldConfig;

  /**
   * Metadata of the article received from WoodWing Studio.
   *
   * @var \Drupal\ww_publish\Metadata
   */
  private $articleMetadata;

  /**
   * Article Components.
   *
   * @var array
   */
  private $articleComponents;

  /**
   * Prepared files.
   *
   * @var array
   */
  private $preparedFiles;

  /**
   * WoodWing Studio article ID field.
   *
   * @var string
   */
  private $articleIdField;

  /**
   * Configuration of the ww_publish module.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $config;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Constructor.
   *
   * @param \Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList $field_config
   *   Field config.
   * @param \Drupal\ww_publish\Metadata $article_metadata
   *   Article components.
   * @param array $article_components
   *   Article data.
   * @param array $prepared_files
   *   Prepared files.
   * @param string $article_id_field
   * @param \Drupal\Core\Config\Config $config
   *   Configuration of the ww_publish module.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   */
  public function __construct($field_config, $article_metadata, $article_components, $prepared_files, $article_id_field, $config, $logger) {
    $this->fieldConfig = $field_config;
    $this->articleMetadata = $article_metadata;
    $this->articleComponents = $article_components;
    $this->preparedFiles = $prepared_files;
    $this->articleIdField = $article_id_field;
    $this->config = $config;
    $this->logger = $logger;
  }

  /**
   * Get article data.
   *
   * @param \Drupal\Core\Field\FieldItemList $messagesField
   *   Error messages field.
   *
   * @return array
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function getArticleData(&$messagesField = NULL): array {

    $identifiers = $this->getIdentifiers();
    $articleData = [];

    foreach ($this->articleComponents as $wwComponent) {
      $identifierKeys = array_keys(
        array_column($identifiers, 'first_identifier'),
        $wwComponent->identifier
      );

      if (!empty($identifierKeys)) {
        $identifierData = [];
        foreach ($identifierKeys as $identifierKey) {
          $identifierData[] = $identifiers[$identifierKey];
        }
        $paragraph = $this->createParagraph($wwComponent, $identifierData, $messagesField);

        if ($paragraph) {
          $articleData[] = [
            'target_id' => $paragraph->id(),
            'target_revision_id' => $paragraph->getRevisionId(),
          ];
        }
      }
    }

    if ($this->config->get('debug_mode'))
      $this->logger->debug('Paragraph article data: <pre><code>' . print_r($articleData, TRUE) . '</code></pre>');
    return $articleData;
  }

  /**
   * Get content of the component.
   *
   * @param object $wwComponent
   *   WoodWing Studio component.
   * @param array $identifiers
   *   The array items are arrays with the following keys: 'identifier',
   *   'first_identifier', 'last_identifier', 'paragraph_type', 'field_name',
   *   'field_type', 'field_config'.
   * @param \Drupal\Core\Field\FieldItemList $messagesField
   *   Error messages field.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  private function createParagraph(object $wwComponent, array $identifiers, &$messagesField = NULL): ?EntityInterface {

    $paragraphData = [
      'type' => $identifiers[0]['paragraph_type'],
    ];

    $createParagraph = FALSE;

    foreach ($identifiers as $identifier) {
      $wwIdentifiers = explode('|', $identifier['identifier']);

      switch ($identifier['field_type']) {
        case 'entity_reference':
          $entityType = $this->getEntityType($identifier['field_config']);
          $referenceId = $this->getReferenceId($entityType, $wwComponent);
          if ($referenceId) {
            $paragraphData[$identifier['field_name']] = [
              'target_id' => $referenceId,
            ];
            $createParagraph = TRUE;
          }
          break;

        case 'entity_reference_revisions':
          if ($this->config->get('debug_mode')) {
            $this->logger->debug('WoodWing Studio component containers for ' . $wwComponent->identifier . ': <pre><code>' . print_r($wwComponent->containers, TRUE) . '</code></pre>');
            $this->logger->debug('Last identifier: ' . $identifier['last_identifier']);
          }
          $paragraphField = new ParagraphField($identifier['field_config'], $this->articleMetadata, $wwComponent->containers->{$identifier['last_identifier']}, $this->preparedFiles, $this->articleIdField, $this->config, $this->logger);
          $articleData = $paragraphField->getArticleData($messagesField);
          if ($articleData) {
            $paragraphData[$identifier['field_name']] = $articleData;
            $createParagraph = TRUE;
          }
          break;

        case 'string':
        case 'string_long':
          $componentContent = $this->getTextComponentContent($wwIdentifiers, $wwComponent, $identifier);

          if ($componentContent) {
            $componentContent = strip_tags($componentContent);
            $paragraphData[$identifier['field_name']] = $componentContent;
            $createParagraph = TRUE;
          }
          break;

        case 'text':
        case 'text_long':
          $componentContent = $this->getTextComponentContent($wwIdentifiers, $wwComponent, $identifier);

          if ($componentContent) {
            $paragraphData[$identifier['field_name']] = $componentContent;
            $createParagraph = TRUE;
          }
          break;

        case 'link':
          $uri = $this->getComponentContent($wwComponent);

          if (!is_string($uri) && $identifier['field_config']->isRequired()) {
            $message = 'The component with identifier ' . $identifier['identifier'] . ' can not be saved as a link in the field ' . $identifier['field_name'] . ' in the paragraph type ' . $identifier['paragraph_type'] . '.';
            $this->logger->error($message . ' Code of the component: <pre><code>' . print_r($wwComponent, TRUE) . '</code></pre>');
            $messagesField[]['value'] = $message;
            break;
          }

          if ($uri) {
            $paragraphData[$identifier['field_name']] = [
              'uri' => $uri,
            ];
            $createParagraph = TRUE;
          }
          break;

        default:
          // Get content from the text property.
          $content = '';
          if (property_exists($wwComponent, 'content') && property_exists($wwComponent->content, 'text')) {
            foreach ($wwComponent->content->text as $text) {
              $content .= $this->getAtributeContent($text);
            }
          }

          if (!empty($content)) {
            // Apply a template on the content.
            $template = new Template($identifier['identifier'], 'component', $this->logger);
            if ($template->exist()) {
              $content = $template->apply(['insert' => $content]);
            }

            // Create a new paragraph.
            $paragraphData[$identifier['field_name']] = [
              'value' => $content,
              'format' => 'full_html',
            ];

            $createParagraph = TRUE;
          }
      }
    }

    if ($createParagraph) {
      if ($this->config->get('debug_mode'))
        $this->logger->debug('Paragraph data: <pre><code>' . print_r($paragraphData, TRUE) . '</code></pre>');

      $paragraph = Paragraph::create($paragraphData);
      $paragraph->save();

      return $paragraph;
    } else {
      return NULL;
    }
  }

  /**
   * Get type of the referenced entity.
   *
   * @param \Drupal\field\Entity\FieldConfig $fieldConfig
   *   Field configuration.
   *
   * @return string
   */
  private function getEntityType(FieldConfig $fieldConfig): ?string {
    $entityType = NULL;
    if (
      $fieldConfig->getSetting('handler') === 'default:media' &&
      $fieldConfig->getSetting('handler_settings') &&
      count($fieldConfig->getSetting('handler_settings')['target_bundles']) === 1
    ) {
      $entityType = reset($fieldConfig->getSetting('handler_settings')['target_bundles']);
    }
    return $entityType;
  }

  /**
   * Get reference ID.
   *
   * @param string $entityType
   *   Type of the referenced entity.
   * @param Object $component
   *   Component.
   *
   * @return integer
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function getReferenceId($entityType, $component): ?int {
    switch ($entityType) {
      case 'image':
        $image = new Image($component->content->image->id, $this->articleMetadata, $this->preparedFiles, $this->articleIdField, $this->config, $this->logger);
        if ($mediaEntity = $image->getMediaEntity()) {
          return $mediaEntity->id();
        } else {
          $this->logger->error('Image media entity could not be created for the component: <pre><code>' . print_r($component, TRUE) . '</code></pre>');
          return NULL;
        }

        break;
    }

    return NULL;
  }

  /**
   * Get the component content.
   *
   * @param object $wwComponent
   *   WoodWing Studio component, from which the content should be extracted.
   * @param string $contentProperty
   *   Property in the content object.
   * @param string $textProperty
   *   Property in the object in the text array.
   *
   * @return string|bool
   */
  private function getComponentContent(object $wwComponent, $contentProperty = 'text', $textProperty = 'insert') {
    $componentContent = [];
    if (property_exists($wwComponent, 'content') && property_exists($wwComponent->content, $contentProperty)) {
      foreach ($wwComponent->content->{$contentProperty} as $text) {
        $componentContent[] = $this->getAtributeContent($text);
      }
      return implode('', $componentContent);
    } else {
      return FALSE;
    }
  }

  /**
   * Get all defined WoodWing Studio identifiers.
   *
   * All identifiers from fields in all enabled paragraph types is returned as
   * an array, of arrays with the following keys: identifier, paragraph_type,
   * field_name.
   *
   * @return array
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  private function getIdentifiers(): array {
    $identifiers = [];
    // Get all enabled paragraph types.
    $enabledParagraphTypes = $this->fieldConfig->get('settings')['handler_settings']['target_bundles'];
    foreach ($enabledParagraphTypes as $type) {
      $fields = \Drupal::service('entity.manager')->getFieldDefinitions('paragraph', $type);
      foreach ($fields as $field_name => $fieldConfig) {
        if ($fieldConfig instanceof FieldConfig && $fieldConfig->get('entity_type') === 'paragraph') {
          $identifier = $fieldConfig->getThirdPartySetting('ww_publish', 'content_station_identifier');
          $paragraph_type = $fieldConfig->get("bundle");
          $explodedIdentifier = explode('|', $identifier);
          $lastIdentifier = end($explodedIdentifier);
          $identifiers[] = [
            'identifier' => $identifier,
            'first_identifier' => strtok($identifier, '|'),
            'last_identifier' => $lastIdentifier,
            'paragraph_type' => $paragraph_type,
            'field_name' => $field_name,
            'field_type' => $fieldConfig->getType(),
            'field_config' => $fieldConfig,
          ];
        }
      }
    }
    return $identifiers;
  }

  /**
   * Get attribute content.
   *
   * @param Object $text
   *   The text object with or without attributes (formatting).
   *
   * @return string
   */
  private function getAtributeContent ($text): string {
    $attributeContent = $text->insert;
    if (property_exists($text, 'attributes')) {
      foreach ($text->attributes as $attribute => $attributeValue) {
        $template = new Template($attribute, 'attribute', $this->logger);
        if ($template->exist()) {
          $attributeContent = $template->apply([
            'insert' => $attributeContent,
            $attribute => $attributeValue,
          ]);
        }
      }
    }
    return $attributeContent;
  }

  /**
   * Get text component.
   */
  private function getTextComponentContent ($wwIdentifiers, $wwComponent, $identifier) {
    $componentContent = FALSE;
    if (count($wwIdentifiers) === 1) {
      $componentContent = $this->getComponentContent($wwComponent);
    }
    if (count($wwIdentifiers) === 2) {
      $componentContent = $this->getComponentContent($wwComponent, $wwIdentifiers[1]);
    }

    if (!is_string($componentContent) && $identifier['field_config']->isRequired()) {
      $message = 'The component with identifier ' . $identifier['identifier'] . ' can not be saved as a string in the field ' . $identifier['field_name'] . ' in the paragraph type ' . $identifier['paragraph_type'] . '.';
      $this->logger->error($message . ' Code of the component: <pre><code>' . print_r($wwComponent, TRUE) . '</code></pre>');
      $this->logger->error('Component content: <pre><code>' . print_r($componentContent, TRUE) . '</code></pre>');
      $messagesField[]['value'] = $message;
      $componentContent = FALSE;
    }

    return $componentContent;
  }
}
