<?php

namespace Drupal\ww_publish;

use \Drupal\ww_publish\Entity\TemplateEntity;

class Template {

  /**
   * Template entity.
   *
   * @var \Drupal\ww_publish\Entity\TemplateEntity
   */
  private $templateEntity;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Constructor.
   *
   * @param string $target
   *   Identifier of the target WoodWing Studio component.
   * @param string $type
   *   Type of the template: component, attribute.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   */
  public function __construct($target, $type, $logger) {
    $this->logger = $logger;
    $query = \Drupal::entityQuery('ww_publish_template');
    $query->condition('target', $target);
    $query->condition('type', $type);
    $templateEntityIds = $query->execute();
    if (!empty($templateEntityIds)) {
      $this->templateEntity = TemplateEntity::load(reset($templateEntityIds));
    } else {
      $this->logger->warning('No template entity found with target: ' . $target . ' and type: ' . $type . '.');
    }
  }

  /**
   * Apply template.
   *
   * @param array $parameters
   *   Parameters used in the template.
   *
   * @return string
   */
  public function apply(array $parameters): string {
    if ($this->templateEntity) {
      $template = $this->templateEntity->template;
      $templateObjects = $this->parse($template);
      $text = '';
      foreach ($templateObjects as $templateObject) {
        foreach ($parameters as $parameter => $content) {
          if ($templateObject->dynamical && $parameter === $templateObject->parameter && !$templateObject->functions) {
            $text .= $content;
          } elseif ($templateObject->dynamical && $parameter === $templateObject->parameter) {
            foreach ($templateObject->functions as $function) {
              $content = $this->applyFunction($content, $function);
            }
            $text .= $content;
          } elseif (!$templateObject->dynamical && !$templateObject->applied) {
            $text .= $templateObject->content;
            $templateObject->applied = TRUE;
          }
        }
      }
      return $text;
    } else {
      $this->logger->error('There is no template entity defined.');
      return '';
    }
  }

  /**
   * Is the template defined?
   *
   * @return bool
   */
  public function exist() {
    if ($this->templateEntity) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  /**
   * Parse the template.
   *
   * @param string $template
   *   Template to be parsed.
   *
   * @return array
   */
  private function parse(string $template): array {
    $splittedTemplate = preg_split("/({{.*}})/U", $template, -1, PREG_SPLIT_DELIM_CAPTURE);
    $templateObjects = [];
    foreach ($splittedTemplate as $templateSnippet) {
      if (str_starts_with($templateSnippet, '{{')) {
        $parameterAndFunctions = explode('::', trim(str_replace(['{{', '}}'], '', $templateSnippet)));
        $templateObjects[] = (object) [
          'dynamical' => TRUE,
          'parameter' => reset($parameterAndFunctions),
          'functions' => count($parameterAndFunctions) > 1 ? array_slice($parameterAndFunctions, 1) : FALSE,
        ];
      } else {
        $templateObjects[] = (object) [
          'dynamical' => FALSE,
          'content' => $templateSnippet,
          'applied' => FALSE,
        ];
      }
    }
    return $templateObjects;
  }

  /**
   * Apply a function.
   *
   * @param string $content
   *   Content which should be updated.
   * @param string $function
   *   Name of the function from the template object.
   *
   * @return string
   */
  private function applyFunction(string $content, string $function): string {
    if ($function === 'toLower') {
      return strtolower($content);
    }

    if ($function === 'toUpper') {
      return strtoupper($content);
    }

    if (str_starts_with($function, 'replace') && preg_match('/\((.*?)\)/U', $function, $match) === 1){
      $replaceObject = json_decode($match[1]);
      if (
        property_exists($replaceObject, 'search') &&
        property_exists($replaceObject, 'replace')
      ) {
        return str_replace($replaceObject->search, $replaceObject->replace, $content);
      }
    }

    return $content;
  }
}
